/*****************************************************************************
  ******* VIC Software 1999 *******
  PCXRead  -  by Wenderson Teixeira
  *********************************

  PCXRead e um programa que demonstra a utilizacao de funcoes simples
  p/ 'mode 13h', 320x200x8, a leitura de teclas extendidas, setas,
  Ctrl+<ch>, etc, e o mais importante, leitura e visualizacao de
  arquivos PCX de 256 cores, nao foi implementado a leitura de
  outros formatos.
  E importante frisar que a leitura de teclas, deve ser substituida,
  principalmente p/ aqueles que querem fazer um jogo, e deve ser
  utilizado um Handler de interrupcao, na 'int 9', p/ tornar a leitura
  mais rapida, processar o pressionamento de multiplas teclas ao mesmo
  tempo e evitar o irrante beep quando se mantem a tecla pressionada.
  As funcoes de 'mode 13h' utilizam DoubleBuffer, caso nao consigam
  alocar memoria, sera escrito direto p/ a memoria de video, o efeito
  e horrivel, mas foi a solucao que eu encontrei p/ debugar em BC3.1.
  Caso se esteja utilizando um bitmap do tamanho da tela ou menor e
  sem Zoom, a funcao ClearScreen nao tera mais sentido de ser
  executada a cada iteracao, ja que toda vez a tela sera totalmente
  reescrita, otimizando-se ainda mais o tempo p/ a visualizacao.
  Retirando-se o Zoom, a escrita em tela aumenta em muito a velcidade.
  Deve-se chamar SwapBuffer toda vez que se quiser atualizar a tela
  com o conteudo do buffer secundario, SwapBuffer so atualiza depois
  que o retraco vertical do monitor e completado.
*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <alloc.h>
#include <dos.h>
#include <mem.h>
#include "pcx.h"
#include "mode13.h"
#include "readkey.h"

#define STEP 5

//----------------------------------------------------------------------------
void ShowPCX(TPCX *pcx, int left, int top, int zoom)
{
  int x, y, width = PCXWidth(pcx), height = PCXHeight(pcx);

  for(y = 0; (y + top) < height && (y / zoom) < SCREENHEIGHT; y++)
    for(x = 0; (x + left) < width && (x / zoom) < SCREENWIDTH; x++)
      PutPixel(x / zoom, y / zoom,
               *(pcx->pbits + (unsigned long)width * (top + y) + (left + x)));
}

//----------------------------------------------------------------------------
int main(int ArgC, char *ArgV[])
{
  TPCX *pcx;
  int x, y, zoom;
  bool done;

  if(ArgC < 2)
  {
    puts("Uso: readpcx arquivo.pcx");
    puts("  Use as teclas Left, Right, Up, Down, Home, PageUp, PageDown, "
         "Ctrl+Left e Ctrl+Right para mover e as teclas + e - para zoom.\n"
         "ESC sai do programa.");
    return EXIT_FAILURE;
  }

  pcx = LoadPCX(ArgV[1]);
  if(!pcx)
  {
    puts("Erro na leitura do arquivo.");
    return EXIT_FAILURE;
  }

  InitMode13();

  x = y = 0;
  zoom = 1;

  BlackOut();
  ShowPCX(pcx, x, y, zoom);
  SwapBuffer();
  FadeIn(&pcx->pal);
  SetPalette(&pcx->pal);

  done = false;
  while(!done)
  {
    int key;

		ClearScreen(0);
    ShowPCX(pcx, x, y, zoom);
    SwapBuffer();

    gotoxy(1, 25);
    printf("(%d, %d)", x, y);

    key = ReadKeyWait();
    switch(key)
    {
      case HOME:
        x = y = 0;
      break;

      case RIGHT:
        x < ((PCXWidth(pcx) / zoom) - SCREENWIDTH) ? x += STEP : 0;
        printf(" > ");
      break;

      case LEFT:
        x >= STEP ? x -= STEP : 0;
        printf(" < ");
      break;

      case DOWN:
        y < ((PCXHeight(pcx) / zoom) - SCREENHEIGHT) ? y += STEP : 0;
        printf(" v ");
      break;

      case UP:
        y >= STEP ? y -= STEP : 0;
        printf(" ^ ");
      break;

      case CTRLRIGHT:
				x < ((PCXWidth(pcx) / zoom) - SCREENWIDTH) ? x += STEP * 10 : 0;
        printf(" >> ");
      break;

      case CTRLLEFT:
        x >= (STEP * 10) ? x -= STEP * 10 : 0;
        printf(" << ");
      break;

      case PAGEDN:
        y < ((PCXHeight(pcx) / zoom) - SCREENHEIGHT) ? y += STEP * 10 : 0;
        printf(" vv ");
      break;

      case PAGEUP:
        y >= (STEP * 10) ? y -= STEP * 10 : 0;
        printf(" ^^ ");
      break;

      case '+':
        zoom > 1 ? zoom -= 1 : 0;
        printf(" + ");
      break;

      case '-':
        zoom < 10 ? zoom += 1 : 0;
        printf(" - ");
      break;

      case ESC:
        done = true;
        FadeOut(&pcx->pal);
      break;
    }
  }

  FreePCX(pcx);

  RestoreOldVideoMode();

  return EXIT_SUCCESS;
}
//----------------------------------------------------------------------------

