
#include <dos.h>
#include <mem.h>
#include <alloc.h>
#include <mode13.h>

//----------------------------------------------------------------------------
int oldVideoMode = 3;
bool usingDoubleBuffer = true;
pfbyte pVideoMemory = (pfbyte)0xA0000000L;
pfbyte pDoubleBuffer = 0;

//----------------------------------------------------------------------------
#ifndef PutPixel
void PutPixel(unsigned x, unsigned y, byte color)
{
	if(x < SCREENWIDTH && y < SCREENHEIGHT)
		*(pDoubleBuffer + y * SCREENWIDTH + x) = color;
}
#endif

//----------------------------------------------------------------------------
bool InitMode13(void)
{
	union REGS r;

	r.h.ah = 0xF;
	int86(0x10, &r, &r);
	oldVideoMode = r.h.al;

	r.x.ax = 0x13;
	int86(0x10, &r, &r);

	pDoubleBuffer = (pfbyte)malloc(sizeof(byte) * 64000U);
	if(!pDoubleBuffer)
	{
		usingDoubleBuffer = false;
		pDoubleBuffer = pVideoMemory;
	}
	else
		usingDoubleBuffer = true;

	ClearScreen(0);

	return usingDoubleBuffer;
}

//----------------------------------------------------------------------------
void RestoreOldVideoMode(void)
{
	union REGS r;

	r.x.ax = oldVideoMode;
	int86(0x10, &r, &r);

	if(usingDoubleBuffer)
		free(pDoubleBuffer);
}

//----------------------------------------------------------------------------
void ClearScreen(byte color)
{
	memset(pDoubleBuffer, color, 64000U);
}

//----------------------------------------------------------------------------
void WaitForRetrace(void)
{
	/* wait until done with vertical retrace */
	while((inp(INPUT_STATUS) & VRETRACE));
	/* wait until done refreshing */
	while(!(inp(INPUT_STATUS) & VRETRACE));
}

//----------------------------------------------------------------------------
void SwapBuffer(void)
{
	if(usingDoubleBuffer)
	{
		WaitForRetrace();
		memcpy(pVideoMemory, pDoubleBuffer, 64000U);
	}
}

//----------------------------------------------------------------------------
void SetPalette(TPalette *pal)
{
	int c;

	outp(PALETTE_COUNTER, pal->count - 1);
	outp(PALETTE_INDEX, 0);
	for(c = 0; c < pal->count; c++)
	{
		outp(PALETTE_DATA, pal->entries[c].red >> 2);
		outp(PALETTE_DATA, pal->entries[c].green >> 2);
		outp(PALETTE_DATA, pal->entries[c].blue >> 2);
	}
}

//----------------------------------------------------------------------------
void BlackOut(void)
{
	int c;

	outp(PALETTE_COUNTER, 255);
	outp(PALETTE_INDEX, 0);
	for(c = 0; c < 256; c++)
	{
		outp(PALETTE_DATA, 0);
		outp(PALETTE_DATA, 0);
		outp(PALETTE_DATA, 0);
	}
}

//----------------------------------------------------------------------------
void FadeIn(TPalette *pal)
{
	int c, i;

	for(i = 1; i < 32; i++)
	{
		outp(PALETTE_COUNTER, pal->count - 1);
		outp(PALETTE_INDEX, 0);
		for(c = 0; c < pal->count; c++)
		{
			outp(PALETTE_DATA, ((pal->entries[c].red >> 5) * i) >> 2);
			outp(PALETTE_DATA, ((pal->entries[c].green >> 5) * i) >> 2);
			outp(PALETTE_DATA, ((pal->entries[c].blue >> 5) * i) >> 2);
		}

		delay(100);
	}
}

//----------------------------------------------------------------------------
void FadeOut(TPalette *pal)
{
	int c, i;

	for(i = 32; i > 0; i--)
	{
		outp(PALETTE_COUNTER, pal->count - 1);
		outp(PALETTE_INDEX, 0);
		for(c = 0; c < pal->count; c++)
		{
			outp(PALETTE_DATA, ((pal->entries[c].red >> 5) * i) >> 2);
			outp(PALETTE_DATA, ((pal->entries[c].green >> 5) * i) >> 2);
			outp(PALETTE_DATA, ((pal->entries[c].blue >> 5) * i) >> 2);
		}

		delay(100);
	}
}

//----------------------------------------------------------------------------

