
#include <alloc.h>
#include <pcx.h>

//----------------------------------------------------------------------------
/* This procedure reads one encoded block from the image file
   and stores a count and data byte.
   Return result:
       true = valid data stored,
       false = out of data in file */

bool encget(short *pByte, short *pCount, FILE *fSource)
{
  int i = getc(fSource);
  *pCount = 1;      /* assume a "run" length of one */
  if(EOF == i)
    return false;

  if(0xC0 == (0xC0 & i))
  {
    *pCount = 0x3F & i;
    if(EOF == (i = getc(fSource)))
      return false;
  }

  *pByte = i;
  return true;
}

//----------------------------------------------------------------------------
TPCX *LoadPCX(const char *filename)
{
  TPCX *pcx;
  FILE *fp = fopen(filename, "rb");

  if(!fp)
    return 0;

  pcx = ReadPCXFile(fp);

  fclose(fp);

  return pcx;
}

//----------------------------------------------------------------------------
void FreePCX(TPCX *pcx)
{
  if(pcx)
  {
    if(pcx->pbits)
			farfree(pcx->pbits);

		farfree(pcx);
  }
}

//----------------------------------------------------------------------------
bool ReadPCXHeader(TPCXHeader *header, FILE *fp)
{
  return fread(header, sizeof(TPCXHeader), 1, fp) == 1;
}

//----------------------------------------------------------------------------
TPCX *ReadPCXFile(FILE *fp)
{
  short chr, cnt;
  int i;
  long l, lsize;
  phbyte pbits;
  TPCX *pcx;

	pcx = (TPCX *)farmalloc(sizeof(TPCX));
  if(!pcx)
    return 0;

  if(!ReadPCXHeader(&pcx->header, fp))
  {
		farfree(pcx);
    return 0;
  }

  lsize = (long)pcx->header.BytesPerLine * pcx->header.NPlanes *
          (1 + pcx->header.YMax - pcx->header.YMin);

	pcx->pbits = (phbyte)farmalloc(sizeof(byte) * lsize);
	if(!pcx->pbits)
	{
		farfree(pcx);
		return 0;
  }

  pbits = pcx->pbits;
  for(l = 0; l < lsize; )             /* increment by cnt below */
  {
    if(!encget(&chr, &cnt, fp))
    {
			farfree(pcx);
      return 0;
      //break;
    }

    for(i = 0; i < cnt; i++)
      *pbits++ = chr;

    l += cnt;
  }

  if(getc(fp) == 12)
  {
    pcx->pal.count = 0;
    while(!feof(fp) && pcx->pal.count < 256)
      if(fread(&pcx->pal.entries[pcx->pal.count++], sizeof(TColor), 1, fp) != 1)
        break;
  }

  return pcx;
}

//----------------------------------------------------------------------------
short PCXWidth(TPCX *pcx)
{
  return pcx->header.XMax - pcx->header.XMin + 1;
}

//----------------------------------------------------------------------------
short PCXHeight(TPCX *pcx)
{
  return pcx->header.YMax - pcx->header.YMin + 1;
}

//----------------------------------------------------------------------------

